/*
 * Copyright 2002-2005 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.sojo.core.conversion.interceptor;

import net.sf.sojo.core.ConverterInterceptor;
import net.sf.sojo.core.reflect.ReflectionHelper;
import net.sf.sojo.util.ThrowableWrapper;

/**
 * Transform before convert the <code>Throwable</code> instance to a <code>ThrowableWrapper</code>
 * and after convert back to a <code>Throwable</code>.
 * 
 * @author linke
 *
 */
public class ThrowableConverterInterceptor implements ConverterInterceptor {

	@Override
	public Object beforeConvert(final Object pvConvertObject, final Class<?> pvToType) {
		if (pvConvertObject instanceof Throwable) {
			return new ThrowableWrapper ((Throwable) pvConvertObject);
		} else {
			return pvConvertObject;
		}
	}

	@Override
	public Object afterConvert(final Object pvResult, final Class<?> pvToType) {
		if (pvResult instanceof ThrowableWrapper) {
			return ReflectionHelper.createThrowable((ThrowableWrapper) pvResult, pvToType);
		} else {
			return pvResult;
		}
	}

	@Override
	public void onError(Exception pvException) {
		// do nothing
	}
}
